require "util"
local game_ready = true

-- search exsisting energy things
function init_energy() 
	global.energy = {}
	global.energy_sources = {"generator", "solar-panel", "reactor", "accumulator"}
	for _, source in ipairs(global.energy_sources) do
		global.energy[source] = {}
		local ents = game.surfaces["nauvis"].find_entities_filtered({type = source})
		for _, ent in pairs(ents) do
			local xy = tostring(ent.position.x) .. tostring(ent.position.y)
			global.energy[source][xy] = ent
		end
	end
end

local function has_value (tab, val)
    for index, value in ipairs(tab) do
        if value == val then
            return true
        end
    end
    return false
end

function add_energy_entity(entity)
	if has_value(global.energy_sources, entity.type) then
		local xy = tostring(entity.position.x) .. tostring(entity.position.y)
		global.energy[entity.type][xy] = entity
	end
end
function delete_energy_entity(entity)
	if has_value(global.energy_sources, entity.type) then
		local xy = tostring(entity.position.x) .. tostring(entity.position.y)
		global.energy[entity.type][xy] = nil
	end
end

function write_stats(filename, stats)
	local str = ""
	for k, v in pairs(stats) do
		str = str .. k .. " " .. tostring(v["cur"]) .. " / " .. tostring(v["max"]).. "\r\n"
	end
	game.write_file("prod_stats/" .. filename, str)
end

function write_energy()
	local energy = {}
	for _, source in ipairs(global.energy_sources) do
		energy[source] = get_entitys_energy(source)
	end
	write_stats("energy.txt", energy)
end

function get_entitys_energy(source)
	local cur = 0
	local max = 0
	for _, ent in pairs(global.energy[source]) do
			cur = cur + ent.energy
			max = max + ent.electric_buffer_size
	end
	local energy = {cur = cur, max = max}
	return energy
end


script.on_event(defines.events.on_tick, function(event)
	if game_ready then
		init_energy()
		game_ready = false
	end
	if game.tick % 60 == 0 then
		write_energy()
	end
end)
--
--- Entity Related Events
script.on_event({
	defines.events.on_built_entity, 
	defines.events.on_robot_built_entity
	}, function(event)
	add_energy_entity(event.created_entity)
end)

script.on_event({
	defines.events.on_preplayer_mined_item,
	defines.events.on_robot_pre_mined,
	defines.events.on_entity_died
	}, function(event)
    	delete_energy_entity(event.entity)
end)
